// /**
//  * @file
//  * Vue app for cart-field.
//  */

(function ($) {
  $(document).ready(function () {
    siteReady();
  });

  if (document.readyState != 'loading') {
    vueCartApp(".product-full-cart");
    setTimeout(() => {
      vueCartApp(".product-teaser");
    }, 1000);
  } else {
    window.addEventListener("DOMContentLoaded", function () {
      vueCartApp(".product-full-cart");
      setTimeout(() => {
        vueCartApp(".product-teaser");
      }, 1000);
    });
  }
  function vueCartApp(sel) {
    var app;
    const elements = document.querySelectorAll(sel);
    const each = Array.prototype.forEach;
    each.call(elements, function (el, i) {
      let comp_template = el.querySelector('cart-field');
      let comp_template_fav = el.querySelector('favorite-field');
      app = Vue.createApp({ delimiters, comments, el }).use(store).use($cookies);
      if (comp_template && comp_template_fav) {
        app.component("cart-field", {
          template: comp_template,
          delimiters: delimiters,
          props: {
            pid: 0,
            uid: 0,
            variations: false,
            attributes: false,
            settings: false,
          },
          data() {
            return {
              inCart: false,
              ready: false,
              loading: false,
              orderId: false,
              variationId: false,
              oldPrice: false,
              price: false,
              priceNumber: false,
              donation: false,
              note: '',
              attributeCount: false,
              activeIds: [],
              enableIds: [],
              quantity: 1,
              stock: 0,
              stores: [],
            };
          },
          computed: {
            cart() {
              return this.$store.state.cart;
            },
            variationSum() {
              let sum = this.quantity * this.priceNumber;
              return sum.toFixed(2);
            },
            availability() {
              if (!this.settings.status) {
                return 0;
              }
              if (!this.settings.checkstore) {
                return 1;
              }
              if (this.stock < this.settings.qstep) {
                return 0;
              }
              return 1;
            }
          },
          methods: {
            attributeClasses(attr) {
              let classes = ' variations-parameter__item--' + attr.label;
              this.activeIds.includes(attr.id) ? classes += ' variations-parameter__item--active' : classes += '';
              !this.enableIds.includes(attr.id) && !this.enableIds.length == 0 ? classes += ' variations-parameter__item--disabled' : classes += '';
              return classes;
            },
            numberFormat(number, step, decimals = 2, dec_point = ".", thousands_sep = " ") {
              let s_number =
                Math.abs(parseInt((number = (+number || 0).toFixed(decimals)))) + "";
              let len = s_number.length;
              let tchunk = len > 3 ? len % 3 : 0;
              let ch_first = tchunk ? s_number.substr(0, tchunk) + thousands_sep : "";
              let ch_rest = s_number
                .substr(tchunk)
                .replace(/(\d\d\d)(?=\d)/g, "$1" + thousands_sep);
              let ch_last = decimals
                ? dec_point + (Math.abs(number) - s_number).toFixed(decimals).slice(2)
                : "";
              if (step == 1) {
                ch_last = '';
              }
              return ch_first + ch_rest + ch_last;
            },
            NumbersOnly(event) {
              event = (event) ? event : window.event;
              var charCode = (event.which) ? event.which : event.keyCode;
              if ((charCode > 31 && (charCode < 48 || charCode > 57)) && charCode !== 46) {
                event.preventDefault();
              } else {
                return true;
              }
            },
            readQuantity() {
              this.quantity = +this.quantity;
              if (this.settings.checkstore) {
                if (this.quantity > this.stock) {
                  this.quantity = this.stock;
                }
              }
              if (this.quantity < this.settings.qstep) {
                this.quantity = this.settings.qstep;
              }
              if (this.settings.quantity_instead_add) {
                axios
                  .post("/cart/set-product-quantity", {
                    vid: this.orderId,
                    quantity: this.quantity,
                  }).then(
                    (response) => {
                      this.$store.commit("updateCart", response.data);
                    },
                  );
              }
            },
            changeQuantity(sign) {
              switch (sign) {
                case 'plus':
                  this.quantity += this.settings.qstep;
                  if (this.settings.checkstore) {
                    if (this.quantity > this.stock) {
                      this.quantity -= this.settings.qstep;
                    }
                  }
                  break;
                case 'minus':
                  this.quantity -= this.settings.qstep;
                  if (this.settings.quantity_instead_add && this.quantity == 0) {
                    this.removeItem(this.orderId);
                  }
                  if (this.quantity <= 0) {
                    this.quantity += this.settings.qstep;
                  }
                  break;
              }
              if (this.settings.quantity_instead_add) {
                axios
                  .post("/cart/set-product-quantity", {
                    vid: this.orderId,
                    quantity: this.quantity,
                  }).then(
                    (response) => {
                      this.$store.commit("updateCart", response.data);
                    },
                  );
              }
            },
            removeItem(vid) {
              axios.post("/cart/remove-cart-item", { vid: vid }).then((response) => {
                this.inCart = false;
                this.initCart();
              });
            },
            addToCart(vid) {
              if (this.loading || !this.variationId || (this.settings.donation && this.donation < 1)) {
                return false;
              }
              this.loading = true;
              axios.post("/cart/add-item", {
                vid: vid,
                qty: this.quantity,
                donation: this.donation,
              }).then(
                (response) => {
                  this.inCart = true;
                  this.loading = false;
                  let items = Object.values(response.data.items)
                  items.forEach(element => {
                    if (element.vid == vid && element.quantity > 0) {
                      this.orderId = element.id;
                    }
                  });
                  this.$store.commit("updateCart", response.data);
                },
              );
            },
            selectVariationId(variation) {
              this.variationId = variation.variation_id;
              this.oldPrice = variation.oldprice_format;
              this.price = variation.price_format;
              this.priceNumber = variation.price;
              this.stock = variation.stock;
              this.stores = variation.stores;
              if (this.settings.checkstore) {
                if (this.quantity > variation.stock) {
                  this.quantity = variation.stock;
                }
              }
            },
            selectVariation(id, name) {
              this.inCart = false;
              this.variationId = 0;
              this.stores = [];
              var xvar = [];
              if (this.settings.masterarray.includes(id)) {
                this.$store.commit("updateColor", id);
                this.enableIds = [];
                this.activeIds = [];
                this.activeIds.push(id);
              }
              else {
                if (!this.activeIds.includes(id)) {
                  for (var attr in this.attributes) {
                    let needclear = false;
                    this.attributes[attr].forEach((v) => {
                      if (id == v.id) {
                        needclear = true;
                      }
                    });
                    if (needclear) {
                      this.attributes[attr].forEach((v) => {
                        if (this.activeIds.includes(v.id)) {
                          this.activeIds.splice(this.activeIds.indexOf(v.id), 1);
                        }
                      });
                      this.activeIds.push(id);
                    }
                  }
                }
              }
              let masterId = 0;
              this.activeIds.forEach((v) => {
                if (this.settings.masterarray.includes(v)) {
                  masterId = v;
                }
              });
              this.variations.forEach((v) => {
                let q = 0;
                v.attributes.forEach((a) => {
                  if (masterId == a.id) {
                    q++;
                  }
                });
                if (q > 0) {
                  v.attributes.forEach((a) => {
                    if (this.settings.masterarray.includes(a.id)) {
                      if (this.settings.checkstore) {
                        if (v.stock > 0) {
                          xvar.push(v);
                        }
                      }
                      else {
                        xvar.push(v);
                      }
                    }
                  });
                }
              });
              this.enableIds = [];
              xvar.forEach((v) => {
                v.attributes.forEach((a) => {
                  if (!this.enableIds.includes(a.id)) {
                    this.enableIds.push(a.id);
                  }
                });
              });
              this.settings.masterarray.forEach((m) => {
                if (!this.enableIds.includes(m)) {
                  this.enableIds.push(m);
                }
              });
              // Убираем "лишнее"
              for (var attr in this.attributes) {
                if (attr != name && !this.settings.masterarray.includes(this.attributes[attr][0].id)) {
                  this.attributes[attr].forEach((v) => {
                    let dis = true;
                    xvar.forEach((x) => {
                      let check = false;
                      x.attributes.forEach((a) => {
                        if (a.id == id) {
                          check = true;
                        }
                      });
                      if (check) {
                        x.attributes.forEach((a) => {
                          if (a.id == v.id) {
                            dis = false;
                          }
                        });
                      }
                    });
                    if (dis) {
                      if (this.enableIds.includes(v.id)) {
                        this.enableIds.splice(this.enableIds.indexOf(v.id), 1);
                      }
                      if (this.activeIds.includes(v.id)) {
                        this.activeIds.splice(this.activeIds.indexOf(v.id), 1);
                      }
                    }
                  });
                }
              }
              if (this.activeIds.length == this.attributeCount) {
                xvar.forEach((v) => {
                  let q = 0;
                  v.attributes.forEach((a) => {
                    if (this.activeIds.includes(a.id)) {
                      q++;
                    }
                  });
                  if (q == this.activeIds.length) {
                    this.oldPrice = v.oldprice_format;
                    this.price = v.price_format;
                    this.priceNumber = v.price;
                    this.stock = v.stock;
                    this.variationId = v.variation_id;
                    this.stores = v.stores;
                    if (this.settings.checkstore) {
                      if (this.quantity > v.stock) {
                        this.quantity = v.stock;
                      }
                    }
                  }
                });
              }
            },
            initCart(vid) {
              let loadPath = "/api/cart-load";
              axios.get(loadPath).then((response) => {
                this.$store.commit("updateCart", response.data);
                if (response.data.quantity > 0) {
                  let items = Object.values(response.data.items)
                  items.forEach(element => {
                    if (element.vid == vid && element.quantity > 0 && element.data.length === 0) {
                      this.orderId = element.id;
                      this.quantity = element.quantity;
                      this.inCart = true;
                    }
                  });
                  this.ready = true;
                }
              });
            },
          },
          mounted() {
            this.ready = true;
            if (typeof this.attributes == "object") {
              this.attributeCount = 0;
              for (var attr in this.attributes) {
                this.attributeCount++;
              }
            }
            let key = 0;
            let teaser = el.closest('.catalog-col');
            if (teaser) {
              let variationId = teaser.getAttribute('data-vid');
              this.variations.forEach((v, i) => {
                if (v.variation_id == variationId) {
                  key = i;
                }
              });
            }
            if (typeof this.variations[key] == 'undefined') {
              return
            }
            this.oldPrice = this.variations[key].oldprice_format;
            this.price = this.variations[key].price_format;
            this.priceNumber = this.variations[key].price;
            this.stock = this.variations[key].stock;
            this.donation = parseInt(this.variations[key].price);
            this.variationId = this.variations[key].variation_id;
            this.stores = this.variations[key].stores;
            if (this.settings.fullstate) {
              this.variations[key].attributes.forEach((v) => {
                if (this.settings.masterarray.includes(v.id)) {
                  this.$store.commit("updateColor", v.id);
                }
                this.selectVariation(v.id, v.label);
              });
            }
            if (this.settings.quantity_instead_add) {
              this.initCart(this.variationId);
            }
            else {
              this.quantity = +this.settings.qstep;
            }
          },
        });
        app.component("favorite-field", {
          template: comp_template_fav,
          delimiters: delimiters,
          props: {
            pid: 0,
          },
          data() {
            return {
              productsLoaded: false,
              fav: false,
            };
          },
          methods: {
            toggleFav() {
              if (this.fav) {
                this.deleteFav();
              } else {
                this.addFav();
              }
            },
            addFav() {
              this.fav = true;
              this.likeClicked = true;
              var favorites = this.$cookies.get("favorites");
              if (typeof favorites !== "object" || favorites == null) {
                favorites = {};
              }
              favorites[this.pid] = true;
              this.$cookies.set("favorites", favorites, "30d");
              var total = Object.keys(favorites).length;
              this.$store.commit("updateFavorite", { total });
            },
            // Удалить из избранное.
            deleteFav() {
              this.fav = false;
              this.likeClicked = false;
              var favorites = this.$cookies.get("favorites");
              if (typeof favorites !== "object" || favorites == null) {
                favorites = {};
              }
              delete favorites[this.pid];
              this.$cookies.set("favorites", favorites, "30d");
              if (window.location.pathname == "/favorites") {
                window.location.reload();
              } else {
                var total = Object.keys(favorites).length;
                this.$store.commit("updateFavorite", { total });
              }
            },
          },
          created() {
            this.productsLoaded = true;
            let favorites = this.$cookies.get("favorites");
            if (
              favorites !== null &&
              typeof favorites == "object" &&
              favorites.hasOwnProperty(this.pid)
            ) {
              this.fav = true;
            }
            if (localStorage.watched) {
              var array = localStorage.watched.split("|");
            } else {
              var array = [];
            }
            let index = array.indexOf(this.pid.toString());
            if (index === -1) {
              array.push(this.pid);
              localStorage.watched = array.join("|");
            }
          },
        });
        app.mount(el);
      }
    });
  }

  String.prototype.replaceAll = function (search, replacement) {
    return this.split(search).join(replacement);
  };

  function siteReady() {
    const END = Date.now() + 3000;
    var body = document.body;

    var interval = setInterval(() => {
      if (body.classList.contains('site-is-ready')) {
        clearInterval(interval);
        var targetNodes = $(".catalog");
        var MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
        var myObserver = new MutationObserver(mutationHandler);
        var obsConfig = {
          childList: true,
          characterData: true,
          attributes: true,
          subtree: true
        };
        targetNodes.each(function () {
          myObserver.observe(this, obsConfig);
        });

        function mutationHandler(mutationRecords) {
          mutationRecords.forEach(function (mutation) {
            setTimeout(() => {
              vueCartApp(".product-teaser");
            }, 1000);
          });
        }
        return;
      }
      if (Date.now() > END) {
        body.classList.add('site-is-ready');
        clearInterval(interval);
      }
    }, 1000);

    window.onload = () => {
      body.classList.add('site-is-ready');
    }
  }
})(jQuery);
